==> ugene_1.5.2+repack.orig.tar.gz: ugene-1.5.2/src/core/src/workflow_support/WorkflowRunTask.cpp <==
/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "WorkflowRunTask.h"
#include <workflow/WorkflowEnv.h>
#include <workflow/WorkflowManager.h>
#include <workflow/Schema.h>
#include <workflow_support/SchemaSerializer.h>

#include <core_api/AppContext.h>
#include <core_api/Log.h>
#include <core_api/Settings.h>
#include <core_api/Counter.h>

namespace GB2 {

WorkflowRunTask::WorkflowRunTask(const Schema& sh, QList<Iteration> lst) :
Task(tr("Execute workflow schema"), TaskFlags(TaskFlag_NoRun) | TaskFlag_ReportingIsSupported) {
    GCOUNTER( cvar, tvar, "WorkflowRunTask" );
    if (lst.isEmpty()) {
        lst.append(Iteration(tr("Main")));
    }
    foreach(const Iteration& it, lst) {
        WorkflowIterationRunTask* t = new WorkflowIterationRunTask(sh, it);
        connect(t, SIGNAL(si_ticked()), SIGNAL(si_ticked()));
        addSubTask(t);
    }
    setMaxParallelSubtasks(MAX_PARALLEL_SUBTASKS_AUTO);
}

QString WorkflowRunTask::generateReport() const {
    QString res;
    res+="<table width='75%'>";
    res+=QString("<tr><th>%1</th><th>%2</th><th>%3</th></tr>").arg(tr("Iterations")).arg(tr("Status")).arg(tr("Details"));
    foreach(Task* sub, getSubtasks()) {
        QString name = Qt::escape(sub->getTaskName());
        QString status = sub->hasErrors() ? tr("Failed") : sub->isCanceled() ? tr("Canceled") : tr("Finished");
        QString error = Qt::escape(sub->getError()).replace("\n", "<br>");
            //AppContext::getTaskScheduler()->getStateName(sub);
        if (sub->hasErrors()) {
            name = "<font color='red'>"+name+"</font>";
            status = "<font color='red'>"+status+"</font>";
        } else if (sub->isCanceled()) {
            status = "<font color='blue'>"+status+"</font>";
        } else {
            status = "<font color='green'>"+status+"</font>";
        }
        res+=QString("<tr><td>%1</td><td>%2</td><td>%3</td></tr>").arg(name).arg(status).arg(error);
    }
    res+="</table>";
    return res;
}

QList<WorkerState> WorkflowRunTask::getState( Actor* actor) {
    QList<WorkerState> ret;
    foreach(Task* t, getSubtasks()) {
        WorkflowIterationRunTask* rt = qobject_cast<WorkflowIterationRunTask*>(t);
        ret << rt->getState(actor);
    }
    return ret;
}

int WorkflowRunTask::getMsgNum( Link* l) {
    int ret = 0;
    foreach(Task* t, getSubtasks()) {
        WorkflowIterationRunTask* rt = qobject_cast<WorkflowIterationRunTask*>(t);
        ret += rt->getMsgNum(l);
    }
    return ret;
}

Task::ReportResult WorkflowRunTask::report() {
    propagateSubtaskError();
    return ReportResult_Finished;
}

WorkflowIterationRunTask::WorkflowIterationRunTask(const Schema& sh, const Iteration& it) : 
Task(QString("%1").arg(it.name), TaskFlags_NR_FOSCOE), schema(new Schema()), scheduler(NULL) {
    rmap = SchemaSerializer::deepCopy(sh, schema);
    schema->applyConfiguration(it, rmap);
}

WorkflowIterationRunTask::~WorkflowIterationRunTask() {
    lmap.clear();
    DomainFactory* df = WorkflowEnv::getDomainRegistry()->getById(schema->domain);
    if (df) {
        df->destroy(scheduler, schema);
    }
    scheduler = NULL;
    delete schema;
}

void WorkflowIterationRunTask::prepare() {
    DomainFactory* df = WorkflowEnv::getDomainRegistry()->getById(schema->domain);
    if (!df) {
        stateInfo.setError(  tr("Unknown domain %1").arg(schema->domain) );
        return;
    }
    foreach(Actor* a, schema->procs) {
        Worker* w = df->createWorker(a);
        if (!w) {
            stateInfo.setError( tr("Failed to create worker %1 %2 in domain %3")\
                .arg(a->getProto()->getId()).arg(a->getId()).arg(schema->domain) );
            return;
        }
    }
    foreach(Link* l, schema->flows) {
        CommunicationChannel* cc = df->createConnection(l);
        if (!cc) {
            stateInfo.setError(  tr("Failed to create connection %1 %2 in domain %3") ); //fixme
            return;
        }
        QStringList lst;
        lst << rmap.key(l->source()->owner()->getId());
        lst << (l->source()->getId());
        lst << rmap.key(l->destination()->owner()->getId());
        lst << (l->destination()->getId());
        QString key = lst.join("|");
        lmap.insert(key, cc);
    }
    scheduler = df->createScheduler(schema, df->getDefaultSettings());
    if (!scheduler) {
        stateInfo.setError(  tr("Failed to create scheduler in domain %1").arg(df->getDisplayName()) );
        return;
    }
    scheduler->init();
    while(scheduler->isReady() && !isCanceled()) {
        Task* t = scheduler->tick();
        if (t) {
            addSubTask(t);
            break;
        }
    }
}

QList<Task*> WorkflowIterationRunTask::onSubTaskFinished(Task* subTask) {
    QList<Task*> tasks;
    if (subTask->hasErrors()) {
        emit si_ticked();
        propagateSubtaskError();
        return tasks;
    }
    while(scheduler->isReady() && !isCanceled()) {
        Task* t = scheduler->tick();
        if (t) {
            tasks << t;
            break;
        }
    }
    emit si_ticked();
    return tasks;
}

Task::ReportResult WorkflowIterationRunTask::report() {
    if (scheduler) {
        scheduler->cleanup();
        if (!scheduler->isDone()) {
            stateInfo.setError( stateInfo.getError() +  tr("\nSchema did not finish.") );
        }
    }
    return ReportResult_Finished;
}

WorkerState WorkflowIterationRunTask::getState(Actor* a)
{
    if (scheduler) {
        return scheduler->getWorkerState(rmap.value(a->getId()));
    }
    return WorkerWaiting;
}

int WorkflowIterationRunTask::getMsgNum( Link* l)
{
    QStringList lst;
    lst << (l->source()->owner()->getId());
    lst << (l->source()->getId());
    lst << (l->destination()->owner()->getId());
    lst << (l->destination()->getId());
    QString key = lst.join("|");
    
    CommunicationChannel* cc = lmap.value(key);
    if (cc) {
        return cc->hasMessage();
    }
    return 0;
}

}//namespace

==> ugene_1.7.1+repack.orig.tar.gz: ugene-1.7.1/src/core/src/workflow_support/WorkflowRunTask.cpp <==
/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include <QtCore/QFile>
#include <QtCore/QFileInfo>

#include "WorkflowRunTask.h"
#include <workflow/WorkflowEnv.h>
#include <workflow/WorkflowManager.h>
#include <workflow/Schema.h>
#include <workflow_support/SchemaSerializer.h>
#include <workflow_support/WorkflowUtils.h>

#include <core_api/Log.h>
#include <core_api/Counter.h>

#define WORKFLOW_RUN_TASK_LOG_CAT "Workflow run task"

namespace GB2 {

static LogCategory log( WORKFLOW_RUN_TASK_LOG_CAT );

/*******************************************
 * WorkflowRunTask
 *******************************************/
WorkflowRunTask::WorkflowRunTask(const Schema& sh, QList<Iteration> lst) :
Task(tr("Execute workflow schema"), TaskFlags(TaskFlag_NoRun) | TaskFlag_ReportingIsSupported) {
    GCOUNTER( cvar, tvar, "WorkflowRunTask" );
    if (lst.isEmpty()) {
        lst.append(Iteration(tr("Main")));
    }
    foreach(const Iteration& it, lst) {
        WorkflowIterationRunTask* t = new WorkflowIterationRunTask(sh, it);
        connect(t, SIGNAL(si_ticked()), SIGNAL(si_ticked()));
        addSubTask(t);
    }
    setMaxParallelSubtasks(MAX_PARALLEL_SUBTASKS_AUTO);
}

QString WorkflowRunTask::generateReport() const {
    QString res;
    res+="<table width='75%'>";
    res+=QString("<tr><th>%1</th><th>%2</th><th>%3</th></tr>").arg(tr("Iterations")).arg(tr("Status")).arg(tr("Details"));
    foreach(Task* sub, getSubtasks()) {
        QString name = Qt::escape(sub->getTaskName());
        QString status = sub->hasErrors() ? tr("Failed") : sub->isCanceled() ? tr("Canceled") : tr("Finished");
        QString error = Qt::escape(sub->getError()).replace("\n", "<br>");
            //AppContext::getTaskScheduler()->getStateName(sub);
        if (sub->hasErrors()) {
            name = "<font color='red'>"+name+"</font>";
            status = "<font color='red'>"+status+"</font>";
        } else if (sub->isCanceled()) {
            status = "<font color='blue'>"+status+"</font>";
        } else {
            status = "<font color='green'>"+status+"</font>";
        }
        res+=QString("<tr><td>%1</td><td>%2</td><td>%3</td></tr>").arg(name).arg(status).arg(error);
    }
    res+="</table>";
    return res;
}

QList<WorkerState> WorkflowRunTask::getState( Actor* actor) {
    QList<WorkerState> ret;
    foreach(Task* t, getSubtasks()) {
        WorkflowIterationRunTask* rt = qobject_cast<WorkflowIterationRunTask*>(t);
        ret << rt->getState(actor);
    }
    return ret;
}

int WorkflowRunTask::getMsgNum( Link* l) {
    int ret = 0;
    foreach(Task* t, getSubtasks()) {
        WorkflowIterationRunTask* rt = qobject_cast<WorkflowIterationRunTask*>(t);
        ret += rt->getMsgNum(l);
    }
    return ret;
}

Task::ReportResult WorkflowRunTask::report() {
    propagateSubtaskError();
    return ReportResult_Finished;
}


/*******************************************
* WorkflowIterationRunTask
*******************************************/
WorkflowIterationRunTask::WorkflowIterationRunTask(const Schema& sh, const Iteration& it) : 
Task(QString("%1").arg(it.name), TaskFlags_NR_FOSCOE), schema(new Schema()), scheduler(NULL) {
    rmap = SchemaSerializer::deepCopy(sh, schema);
    schema->applyConfiguration(it, rmap);
    
    DomainFactory* df = WorkflowEnv::getDomainRegistry()->getById(schema->domain);
    if (!df) {
        stateInfo.setError(  tr("Unknown domain %1").arg(schema->domain) );
        return;
    }
}

WorkflowIterationRunTask::~WorkflowIterationRunTask() {
    lmap.clear();
    DomainFactory* df = WorkflowEnv::getDomainRegistry()->getById(schema->domain);
    if (df) {
        df->destroy(scheduler, schema);
    }
    scheduler = NULL;
    delete schema;
}

void WorkflowIterationRunTask::prepare() {
    if( hasErrors() || isCanceled() ) {
        return;
    }
    
    DomainFactory* df = WorkflowEnv::getDomainRegistry()->getById(schema->domain);
    assert( df != NULL ); // checked in constructor
    foreach(Actor* a, schema->procs) {
        Worker* w = df->createWorker(a);
        if (!w) {
            stateInfo.setError( tr("Failed to create worker %1 %2 in domain %3")\
                .arg(a->getProto()->getId()).arg(a->getId()).arg(schema->domain) );
            return;
        }
    }
    foreach(Link* l, schema->flows) {
        CommunicationChannel* cc = df->createConnection(l);
        if (!cc) {
            stateInfo.setError(  tr("Failed to create connection %1 %2 in domain %3") ); //fixme
            return;
        }
        QStringList lst;
        lst << rmap.key(l->source()->owner()->getId());
        lst << (l->source()->getId());
        lst << rmap.key(l->destination()->owner()->getId());
        lst << (l->destination()->getId());
        QString key = lst.join("|");
        lmap.insert(key, cc);
    }
    scheduler = df->createScheduler(schema, df->getDefaultSettings());
    if (!scheduler) {
        stateInfo.setError(  tr("Failed to create scheduler in domain %1").arg(df->getDisplayName()) );
        return;
    }
    scheduler->init();
    while(scheduler->isReady() && !isCanceled()) {
        Task* t = scheduler->tick();
        if (t) {
            addSubTask(t);
            break;
        }
    }
}

QList<Task*> WorkflowIterationRunTask::onSubTaskFinished(Task* subTask) {
    QList<Task*> tasks;
    if (subTask->hasErrors()) {
        emit si_ticked();
        propagateSubtaskError();
        return tasks;
    }
    while(scheduler->isReady() && !isCanceled()) {
        Task* t = scheduler->tick();
        if (t) {
            tasks << t;
            break;
        }
    }
    emit si_ticked();
    return tasks;
}

Task::ReportResult WorkflowIterationRunTask::report() {
    if (scheduler) {
        scheduler->cleanup();
        if (!scheduler->isDone()) {
            stateInfo.setError( stateInfo.getError() +  tr("\nSchema did not finish.") );
        }
    }
    return ReportResult_Finished;
}

WorkerState WorkflowIterationRunTask::getState(Actor* a)
{
    if (scheduler) {
        return scheduler->getWorkerState(rmap.value(a->getId()));
    }
    return WorkerWaiting;
}

int WorkflowIterationRunTask::getMsgNum( Link* l)
{
    QStringList lst;
    lst << (l->source()->owner()->getId());
    lst << (l->source()->getId());
    lst << (l->destination()->owner()->getId());
    lst << (l->destination()->getId());
    QString key = lst.join("|");
    
    CommunicationChannel* cc = lmap.value(key);
    if (cc) {
        return cc->hasMessage();
    }
    return 0;
}

}//namespace
